import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:ocg_app/ocg_app.dart';

class FirebaseRepository<T> extends Crud<T> {
  final String collection;
  final Future<T> Function(Map<String, dynamic> dynamic) fromJson;
  final Map<String, dynamic> Function(T t) toJson;

  FirebaseRepository(this.collection,
      {@required this.fromJson, @required this.toJson});

  CollectionReference get collectionRefrence =>
      FirebaseFirestore.instance.collection(collection);

  @override
  Future<AppResponse<bool>> delete(String id) async {
    try {
      await collectionRefrence.doc(id).delete();
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<List<T>>> getList({int page = 1, int count = 10}) async {
    try {
      var snapshot = await collectionRefrence.get();
      var data = await Future.wait(
          snapshot.docs.map((e) async => await fromJson(e.data())).toList());
      return AppResponse.named(data: List<T>.from(data));
    } catch (e) {
      return AppResponse.named(error: '$e');
    }
  }

  @override
  Future<AppResponse<T>> read(String id) async {
    try {
      var snapshot = await collectionRefrence.doc(id).get();
      var json = snapshot.data();
      var data = await fromJson(json);
      if (data != null) return AppResponse.named(data: data);
      return AppResponse.named(error: "Not found");
    } catch (e) {
      return AppResponse.named(error: '$e');
    }
  }

  @override
  Future<AppResponse<String>> create(T object) async {
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> update(T object) {
    throw UnimplementedError();
  }
}
